from typing import List


def convert_to_list(value: str) -> List[int]:
    """Convert string to int list

        Args:
            value: String value
        Returns:
            List of integer values decomposed by digits

    """
    return [int(i) for i in value]


def convert_to_str(value: List[int]) -> str:
    """Convert int list to int

        Args:
            value: List consisting of integer
        Returns
            String value combined list of integer elements

    """
    s = [str(i) for i in value]
    res = "".join(s)
    return res



"""Command-line implementation of crc-calculator"""
import sys
from typing import Optional
from typing import Sequence

from crc_calculator.calculator import calculate
from crc_calculator.parser import Parser
from crc_calculator.utils import convert_to_list


def main(argv: Optional[Sequence[str]] = None):
    """Main function to calculate crc.

        Args:
            argv: Arguments vector

    """
    parser = Parser()
    parser.parse(argv or sys.argv[1:])

    polynomial = parser.get_polynomial().lstrip('0')
    data = parser.get_data().lstrip('0')

    checksum = calculate(
        polynomial=convert_to_list(polynomial),
        data=convert_to_list(data),
    )

    validation = calculate(
        polynomial=convert_to_list(polynomial),
        data=convert_to_list(data+checksum),
    )

    print(f'checksum: {checksum}')
    print(f'validation: {validation}')



"""Module for handling cli options"""
from argparse import ArgumentParser
from typing import Optional
from typing import Sequence

from . import __version__


class Parser:
    _parser = ArgumentParser()

    def __init__(self):
        self.args = None

        self._parser.add_argument(
            "-p",
            "--polynomial",
            type=str,
            required=True,
            help="polynomial code that expressed as strings of binary digits",
        )
        self._parser.add_argument(
            "-d",
            "--data",
            type=str,
            required=True,
            help="data that expressed as strings of binary digits",
        )
        self._parser.add_argument(
            "-V",
            "--version",
            action='version',
            version=f'crc-calculator {__version__}',
        )

    def parse(self, argv: Optional[Sequence[str]] = None) -> None:
        """Parse arguments vector

            Args:
                argv: Arguments vector

        """
        self.args = self._parser.parse_args(argv)

    def get_polynomial(self) -> str:
        """Returns polynomial

            Returns:
                polynomial

        """
        polynomial = self.args.polynomial
        validate(polynomial)
        return polynomial

    def get_data(self) -> str:
        """Returns data

            Returns:
                data

        """
        data = self.args.data
        validate(data)
        return data


def validate(value: str) -> None:
    """Check if value is binary.

        Args:
            value: Binary string

    """
    if not all(map(lambda x: x in ["0", "1"], list(value))):
        print("Please enter binary numbers only!")
        exit(0)



from typing import List

from crc_calculator.utils import convert_to_str


def calculate(polynomial: List[int], data: List[int]) -> str:
    """
        Args:
            polynomial: polynomial code
            data: input data
        Returns:
            reminder calculating polynomial and data
    """
    append_zeroes(polynomial, data)

    result = xor(polynomial, data)
    count = len(data) - len(polynomial)

    for i in range(count):
        result.pop(0)
        result.append(data[len(polynomial)+i])

        result = xor(
            polynomial if result[0] != 0 else [
                0 for _ in range(len(polynomial))
            ], result,
        )

    result.pop(0)
    return convert_to_str(result)


def append_zeroes(polynomial: List[int], data: List[int]):
    """Append zeroes to the data by the length of the polynomial"""
    result_length = len(polynomial) - 1
    for i in range(result_length):
        data.append(0)


def xor(a: List[int], b: List[int]) -> List[int]:
    """Calculate xor between int list"""
    result = []

    for i in range(len(a)):
        result.append(a[i] ^ b[i])

    return result
